@@ -1,3 +1,4 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
 from astropy.extern import six
 from .coordinate_helpers import CoordinateHelper
 from .transforms import WCSPixel2WorldTransform
@@ -9,7 +10,8 @@ from .coordinate_range import find_coordinate_range
 class CoordinatesMap(object):
 
     def __init__(self, axes, wcs=None, transform=None, coord_meta=None,
-                 slice=None, frame_class=RectangularFrame):
+                 slice=None, frame_class=RectangularFrame,
+                 previous_frame_path=None):
 
         # Keep track of parent axes and WCS
         self._axes = axes
@@ -30,7 +32,7 @@ class CoordinatesMap(object):
             self._transform = WCSPixel2WorldTransform(wcs, slice=slice)
             naxis = wcs.wcs.naxis
 
-        self.frame = frame_class(axes, self._transform)
+        self.frame = frame_class(axes, self._transform, path=previous_frame_path)
 
         # Set up coordinates
         self._coords = []
@@ -61,7 +63,6 @@ class CoordinatesMap(object):
                                                  coord_unit=coord_unit,
                                                  frame=self.frame))
 
-
             # Set up aliases for coordinates
             self._aliases[name.lower()] = coord_index
 
@@ -112,4 +113,5 @@ class CoordinatesMap(object):
         ymin, ymax = self._axes.get_ylim()
         return find_coordinate_range(self._transform,
                                      [xmin, xmax, ymin, ymax],
-                                     [coord.coord_type for coord in self])
+                                     [coord.coord_type for coord in self],
+                                     [coord.coord_unit for coord in self])
