@@ -1,3 +1,4 @@
+# Licensed under a 3-clause BSD style license - see LICENSE.rst
 import numpy as np
 
 from matplotlib.lines import Path, Line2D
@@ -36,7 +37,9 @@ class Ticks(Line2D):
         self.clear()
         line2d_kwargs = {
             'color': rcParams['xtick.color'],
-            'linewidth': rcParams['xtick.major.width']
+            # For the linewidth we need to set a default since old versions of
+            # matplotlib don't have this.
+            'linewidth': rcParams.get('xtick.major.width', 1)
         }
         line2d_kwargs.update(kwargs)
         Line2D.__init__(self, [0.], [0.], **line2d_kwargs)
@@ -158,6 +161,9 @@ class Ticks(Line2D):
 
         for axis in self.get_visible_axes():
 
+            if not axis in pixel_array:
+                continue
+
             for loc, angle in zip(pixel_array[axis], angle_array[axis]):
 
                 # Set the rotation for this tick
